//
// DESCRIPTION:
//	DOOM graphics stuff for VoCore Display, OpenWrt.
//  based on i_video.c.
//
//-----------------------------------------------------------------------------

#include "includes.h"

#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/select.h>
#include <linux/input.h>
#include <time.h>

#include "doomdef.h"

unsigned int SCREENWIDTH = 320;
unsigned int SCREENHEIGHT = 240;
unsigned int ORIGSCREENWIDTH = 320;
unsigned int ORIGSCREENHEIGHT = 200;

static unsigned short palette_table[256];
static byte *framebuffer;

void I_InitGraphics (void)
{
	int fd = open("/dev/fb0", O_RDWR);
	if (fd < 0) {
		printf("can not open framebuffer.\n");
		exit(-1);
	}

	framebuffer = mmap(NULL, 800 * 480 * 2, PROT_READ|PROT_WRITE,
			   MAP_SHARED, fd, 0);
	if (framebuffer == NULL) {
		printf("can not mmap framebuffer.\n");
		exit(-1);
	}

	system("dd if=/dev/zero of=/dev/fb0 count=1 bs=768000 2> /dev/null");
}

void I_ShutdownGraphics(void)
{
}

void swap_byte(unsigned short *in)
{
	char *p = (char *)in, tmp;
	tmp = p[0];
	p[0] = p[1];
	p[1] = tmp;
}

void I_SetPalette (byte* palette)
{
    unsigned int colour = 0;
    do {
	palette_table[colour] = 0;

#ifdef USE_SCREEN_VOCORE2
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3);
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 2) << 5;
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3) << 11;
	swap_byte(palette_table + colour);
#endif

#ifdef USE_SCREEN_DJN1522
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3) << 11;
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 2) << 5;
	palette_table[colour] |= (((unsigned short)gammatable[usegamma][*palette++]) >> 3);
#endif
    } while (++colour < 256);
}

void I_FinishUpdate (void)
{
    byte *src = screens[0], *dst = framebuffer;
    int i, j, pos;

    int w, h;
    h = SCREENHEIGHT * 4;
    w = SCREENWIDTH * 4;

    for (j = 0; j < h; j += 4) {
	for (i = 0; i < w; i += 4) {
	    pos = 720 * 480 * 2 - i * 480 + j;
	    *(unsigned short *)(dst + pos    ) = palette_table[*src];
	    *(unsigned short *)(dst + pos + 2) = palette_table[*src];
	    *(unsigned short *)(dst + pos - 960) = palette_table[*src];
	    *(unsigned short *)(dst + pos - 958) = palette_table[*src];
	    src++;
	}
    }
    usleep(16000);
}

void I_ReadScreen (byte* scr)
{
    memcpy(scr, screens[0], SCREENWIDTH * SCREENHEIGHT);
}

void I_InitKeyboard (void)
{
}

void I_UpdateNoBlit (void)
{
}

void I_SetScreenSize (void)
{
}

void I_SetWindowName (const char * title)
{
}

void I_StartFrame (void)
{
}

int translate_keyboard(int key)
{
    // use key value to avoid key define conflict.
    switch (key) {
    case 105: return 0xac;
    case 106: return 0xae;
    case 108: return 0xaf;
    case 103: return 0xad;
    case 1: return 27;
    case 28: return 13;
    case 15: return 9;
    case 42: return (0x80+0x36);
    case 54: return (0x80+0x36);
    case 29: return (0x80+0x1d);
    case 97: return (0x80+0x1d);
    case 56: return (0x80+0x38);
    case 100: return (0x80+0x38);
    default: return key;
    }
    return 0;
}

int keyboard = -1;

void I_StartTic (void)
{
    fd_set set;
    struct timeval tv;
    int count, size;

    if (keyboard < 0) {
	keyboard = open("/dev/input/event0", O_RDONLY);
	if (keyboard < 0) {
	    printf("error: can not open keyboard.\n");
	    exit(0);
	}
    }

    tv.tv_sec = 0;
    tv.tv_usec = 1000;

    FD_ZERO(&set);
    FD_SET(keyboard, &set);

    count = select(keyboard + 1, &set, NULL, NULL, &tv);
    if (count <= 0)
	    return;

    if (FD_ISSET(keyboard, &set)) {
	 struct input_event e;
	 event_t event;

	 size = read(keyboard, &e, sizeof(struct input_event));
	 if (size != sizeof(struct input_event))
		 return;
	 if (e.type != EV_KEY)
		 return;
	 event.type = e.value ? ev_keydown : ev_keyup;
	 event.data1 = translate_keyboard(e.code);
	 D_PostEvent(&event);
    }
}

